@@ -15,11 +15,10 @@
 #include "xiterator.hpp"
 #include "xoperation.hpp"
 #include "xmath.hpp"
+#include "xtensor_forward.hpp"
 
 namespace xt
 {
-    template <class C>
-    struct xcontainer_inner_types;
 
     namespace check_policy
     {
@@ -65,14 +64,14 @@ namespace xt
         using shape_type = typename inner_types::shape_type;
         using strides_type = typename inner_types::strides_type;
 
+        using iterator = typename container_type::iterator;
+        using const_iterator = typename container_type::const_iterator;
+
         using stepper = xstepper<D>;
         using const_stepper = xstepper<const D>;
 
-        using iterator = xiterator<stepper, shape_type>;
-        using const_iterator = xiterator<const_stepper, shape_type>;
-
-        using storage_iterator = typename container_type::iterator;
-        using const_storage_iterator = typename container_type::const_iterator;
+        using broadcast_iterator = xiterator<stepper, shape_type*>;
+        using const_broadcast_iterator = xiterator<const_stepper, shape_type*>;
 
         size_type size() const noexcept;
 
@@ -88,8 +87,16 @@ namespace xt
 
         void transpose();
 
-        template <class Tag = check_policy::none>
-        void transpose(const std::vector<size_type>& permutation, Tag check_policy = Tag());
+        template <class S, class Tag = check_policy::none>
+        void transpose(S&& permutation, Tag check_policy = Tag());
+
+#ifdef X_OLD_CLANG
+        template <class I, class Tag = check_policy::none>
+        void transpose(std::initializer_list<I> permutation, Tag check_policy = Tag());
+#else
+        template <class I, std::size_t N, class Tag = check_policy::none>
+        void transpose(const I(&permutation)[N], Tag check_policy = Tag());
+#endif
 
         template <class... Args>
         reference operator()(Args... args);
@@ -122,6 +129,14 @@ namespace xt
         const_iterator cbegin() const noexcept;
         const_iterator cend() const noexcept;
 
+        broadcast_iterator xbegin() noexcept;
+        broadcast_iterator xend() noexcept;
+
+        const_broadcast_iterator xbegin() const noexcept;
+        const_broadcast_iterator xend() const noexcept;
+        const_broadcast_iterator cxbegin() const noexcept;
+        const_broadcast_iterator cxend() const noexcept;
+
         template <class S>
         xiterator<stepper, S> xbegin(const S& shape) noexcept;
         template <class S>
@@ -146,15 +161,6 @@ namespace xt
         template <class S>
         const_stepper stepper_end(const S& shape) const noexcept;
 
-        storage_iterator storage_begin() noexcept;
-        storage_iterator storage_end() noexcept;
-
-        const_storage_iterator storage_begin() const noexcept;
-        const_storage_iterator storage_end() const noexcept;
-
-        const_storage_iterator storage_cbegin() const noexcept;
-        const_storage_iterator storage_cend() const noexcept;
-
     protected:
 
         xcontainer() = default;
@@ -175,8 +181,11 @@ namespace xt
         void adapt_strides() noexcept;
         void adapt_strides(size_type i) noexcept;
 
-        void transpose_impl(const std::vector<size_type>& permutation, check_policy::none);
-        void transpose_impl(const std::vector<size_type>& permutation, check_policy::full);
+        template <class S>
+        void transpose_impl(S&& permutation, check_policy::none);
+
+        template <class S>
+        void transpose_impl(S&& permutation, check_policy::full);
 
         size_type data_size() const noexcept;
 
@@ -392,24 +401,42 @@ namespace xt
 
     /**
      * Transposes the container inplace by permuting the shape with @p permutation.
-     * @param permutation the vector containing permutation
+     * @param permutation the sequence containing permutation
      * @param check_policy the check level (check_policy::full() or check_policy::none())
      * @tparam Tag selects the level of error checking on permutation vector defaults to check_policy::none.
      */
     template <class D>
-    template <class Tag>
-    inline void xcontainer<D>::transpose(const std::vector<size_type>& permutation, Tag check_policy)
+    template <class S, class Tag>
+    inline void xcontainer<D>::transpose(S&& permutation, Tag check_policy)
+    {
+        transpose_impl(std::forward<S>(permutation), check_policy);
+    }
+
+#ifdef X_OLD_CLANG
+    template <class D>
+    template <class I, class Tag>
+    inline void xcontainer<D>::transpose(std::initializer_list<I> permutation, Tag check_policy)
+    {
+        std::vector<I> perm(permutation);
+        transpose_impl(std::move(perm), check_policy);
+    }
+#else
+    template <class D>
+    template <class I, std::size_t N, class Tag>
+    inline void xcontainer<D>::transpose(const I(&permutation)[N], Tag check_policy)
     {
         transpose_impl(permutation, check_policy);
     }
+#endif
 
     template <class D>
-    inline void xcontainer<D>::transpose_impl(const std::vector<size_type>& permutation, check_policy::full)
+    template <class S>
+    inline void xcontainer<D>::transpose_impl(S&& permutation, check_policy::full)
     {
         // check if axis appears twice in permutation
-        for (size_type i = 0; i < permutation.size(); ++i)
+        for (size_type i = 0; i < container_size(permutation); ++i)
         {
-            for (size_type j = i + 1; j < permutation.size(); ++j)
+            for (size_type j = i + 1; j < container_size(permutation); ++j)
             {
                 if (permutation[i] == permutation[j])
                 {
@@ -421,9 +448,10 @@ namespace xt
     }
 
     template <class D>
-    inline void xcontainer<D>::transpose_impl(const std::vector<size_type>& permutation, check_policy::none)
+    template <class S>
+    inline void xcontainer<D>::transpose_impl(S&& permutation, check_policy::none)
     {
-        if (permutation.size() != m_shape.size())
+        if (container_size(permutation) != m_shape.size())
         {
             throw transpose_error("Permutation does not have the same size as shape");
         }
@@ -440,7 +468,7 @@ namespace xt
 
         for (size_type i = 0; i < m_shape.size(); ++i)
         {
-            if (permutation[i] >= dimension())
+            if (size_type(permutation[i]) >= dimension())
             {
                 throw transpose_error("Permutation contains wrong axis");
             }
@@ -500,11 +528,11 @@ namespace xt
     }
 
     /**
-    * Returns a constant reference to the element at the specified position in the container.
-    * @param index a sequence of indices specifying the position in the container. Indices
-    * must be unsigned integers, the number of indices in the list should be equal or greater
-    * than the number of dimensions of the container.
-    */
+     * Returns a constant reference to the element at the specified position in the container.
+     * @param index a sequence of indices specifying the position in the container. Indices
+     * must be unsigned integers, the number of indices in the list should be equal or greater
+     * than the number of dimensions of the container.
+     */
     template <class D>
     inline auto xcontainer<D>::operator[](const xindex& index) const -> const_reference
     {
@@ -598,12 +626,81 @@ namespace xt
      */
     //@{
     /**
-     * Returns an iterator to the first element of the container.
+     * Returns an iterator to the first element of the buffer containing
+     * the elements of the container.
      */
     template <class D>
     inline auto xcontainer<D>::begin() noexcept -> iterator
     {
-        return xbegin(shape());
+        return data().begin();
+    }
+
+    /**
+     * Returns an iterator to the element following the last element of
+     * the buffer containing the elements of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::end() noexcept -> iterator
+    {
+        return data().end();
+    }
+
+    /**
+     * Returns a constant iterator to the first element of the buffer
+     * containing the elements of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::begin() const noexcept -> const_iterator
+    {
+        return data().cbegin();
+    }
+
+    /**
+     * Returns a constant iterator to the element following the last
+     * element of the buffer containing the elements of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::end() const noexcept -> const_iterator
+    {
+        return data().cend();
+    }
+
+    /**
+     * Returns a constant iterator to the first element of the buffer
+     * containing the elements of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::cbegin() const noexcept -> const_iterator
+    {
+        return data().cbegin();
+    }
+
+    /**
+     * Returns a constant iterator to the element following the last
+     * element of the buffer containing the elements of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::cend() const noexcept -> const_iterator
+    {
+        return data().cend();
+    }
+    //@}
+
+    /**************************
+     * broadcast iterator api *
+     **************************/
+
+    /**
+     * @name Broadcast iterators
+     */
+    //@{
+    /**
+     * Returns an iterator to the first element of the container.
+     */
+    template <class D>
+    inline auto xcontainer<D>::xbegin() noexcept -> broadcast_iterator
+    {
+        return broadcast_iterator(stepper_begin(m_shape), &m_shape);
     }
 
     /**
@@ -611,18 +708,18 @@ namespace xt
      * of the container.
      */
     template <class D>
-    inline auto xcontainer<D>::end() noexcept -> iterator
+    inline auto xcontainer<D>::xend() noexcept -> broadcast_iterator
     {
-        return xend(shape());
+        return broadcast_iterator(stepper_end(m_shape), &m_shape);
     }
 
     /**
      * Returns a constant iterator to the first element of the container.
      */
     template <class D>
-    inline auto xcontainer<D>::begin() const noexcept -> const_iterator
+    inline auto xcontainer<D>::xbegin() const noexcept -> const_broadcast_iterator
     {
-        return xbegin(shape());
+        return const_broadcast_iterator(stepper_begin(m_shape), &m_shape);
     }
 
     /**
@@ -630,18 +727,18 @@ namespace xt
      * of the container.
      */
     template <class D>
-    inline auto xcontainer<D>::end() const noexcept -> const_iterator
+    inline auto xcontainer<D>::xend() const noexcept -> const_broadcast_iterator
     {
-        return xend(shape());
+        return const_broadcast_iterator(stepper_end(m_shape), &m_shape);
     }
 
     /**
      * Returns a constant iterator to the first element of the container.
      */
     template <class D>
-    inline auto xcontainer<D>::cbegin() const noexcept -> const_iterator
+    inline auto xcontainer<D>::cxbegin() const noexcept -> const_broadcast_iterator
     {
-        return begin();
+        return xbegin();
     }
 
     /**
@@ -649,9 +746,9 @@ namespace xt
      * of the container.
      */
     template <class D>
-    inline auto xcontainer<D>::cend() const noexcept -> const_iterator
+    inline auto xcontainer<D>::cxend() const noexcept -> const_broadcast_iterator
     {
-        return end();
+        return xend();
     }
 
     /**
@@ -763,75 +860,6 @@ namespace xt
         return const_stepper(static_cast<const derived_type*>(this), data().end(), offset);
     }
 
-    /************************
-     * storage_iterator api *
-     ************************/
-
-    /**
-     * @name Storage iterators
-     */
-    //@{
-    /**
-     * Returns an iterator to the first element of the buffer containing
-     * the elements of the container.
-     */
-    template <class D>
-    inline auto xcontainer<D>::storage_begin() noexcept -> storage_iterator
-    {
-        return data().begin();
-    }
-
-    /**
-     * Returns an iterator to the element following the last element of
-     * the buffer containing the elements of the container.
-     */
-    template <class D>
-    inline auto xcontainer<D>::storage_end() noexcept -> storage_iterator
-    {
-        return data().end();
-    }
-
-    /**
-     * Returns a constant iterator to the first element of the buffer
-     * containing the elements of the container.
-     */
-    template <class D>
-    inline auto xcontainer<D>::storage_begin() const noexcept -> const_storage_iterator
-    {
-        return data().cbegin();
-    }
-
-    /**
-     * Returns a constant iterator to the element following the last
-     * element of the buffer containing the elements of the container.
-     */
-    template <class D>
-    inline auto xcontainer<D>::storage_end() const noexcept -> const_storage_iterator
-    {
-        return data().cend();
-    }
-
-    /**
-     * Returns a constant iterator to the first element of the buffer
-     * containing the elements of the container.
-     */
-    template <class D>
-    inline auto xcontainer<D>::storage_cbegin() const noexcept -> const_storage_iterator
-    {
-        return data().cbegin();
-    }
-
-    /**
-     * Returns a constant iterator to the element following the last
-     * element of the buffer containing the elements of the container.
-     */
-    template <class D>
-    inline auto xcontainer<D>::storage_cend() const noexcept -> const_storage_iterator
-    {
-        return data().cend();
-    }
-    //@}
-
 }
 
 #endif
