@@ -16,9 +16,11 @@
 #include <array>
 #include <algorithm>
 
-#include "xarray.hpp"
-#include "xview_utils.hpp"
+#include "xtensor_forward.hpp"
+#include "xbroadcast.hpp"
 #include "xiterator.hpp"
+#include "xsemantic.hpp"
+#include "xview_utils.hpp"
 
 namespace xt
 {
@@ -27,14 +29,11 @@ namespace xt
      * xview declaration *
      *********************/
 
-    template <class CT, class... S>
-    class xview;
-
     template <class CT, class... S>
     struct xcontainer_inner_types<xview<CT, S...>>
     {
         using xexpression_type = std::decay_t<CT>;
-        using temporary_type = xarray<typename xexpression_type::value_type>;
+        using temporary_type = view_temporary_type_t<xexpression_type, S...>;
     };
 
     template <bool is_const, class CT, class... S>
@@ -83,11 +82,11 @@ namespace xt
         using stepper = xview_stepper<false, CT, S...>;
         using const_stepper = xview_stepper<true, CT, S...>;
 
-        using iterator = xiterator<stepper, shape_type>;
-        using const_iterator = xiterator<const_stepper, shape_type>;
+        using broadcast_iterator = xiterator<stepper, shape_type*>;
+        using const_broadcast_iterator = xiterator<const_stepper, shape_type*>;
         
-        using storage_iterator = iterator;
-        using const_storage_iterator = const_iterator;
+        using iterator = broadcast_iterator;
+        using const_iterator = const_broadcast_iterator;
 
         template <class... SL>
         xview(CT e, SL&&... slices) noexcept;
@@ -95,6 +94,9 @@ namespace xt
         template <class E>
         self_type& operator=(const xexpression<E>& e);
 
+        template <class E>
+        disable_xexpression<E, self_type>& operator=(const E& e);
+
         size_type dimension() const noexcept;
 
         const shape_type& shape() const noexcept;
@@ -126,6 +128,14 @@ namespace xt
         const_iterator cbegin() const;
         const_iterator cend() const;
 
+        broadcast_iterator xbegin();
+        broadcast_iterator xend();
+
+        const_broadcast_iterator xbegin() const;
+        const_broadcast_iterator xend() const;
+        const_broadcast_iterator cxbegin() const;
+        const_broadcast_iterator cxend() const;
+
         template <class ST>
         xiterator<stepper, ST> xbegin(const ST& shape);
         template <class ST>
@@ -150,15 +160,6 @@ namespace xt
         template <class ST>
         const_stepper stepper_end(const ST& shape) const;
 
-        storage_iterator storage_begin();
-        storage_iterator storage_end();
-
-        const_storage_iterator storage_begin() const;
-        const_storage_iterator storage_end() const;
-
-        const_storage_iterator storage_cbegin() const;
-        const_storage_iterator storage_cend() const;
-
     private:
 
         // VS 2015 workaround (yes, really)
@@ -325,7 +326,7 @@ namespace xt
         {
             size_type index = integral_skip<S...>(i);
             m_shape[i] = index < sizeof...(S) ?
-                apply<std::size_t>(index, func, m_slices) : m_e.shape()[index];
+                apply<std::size_t>(index, func, m_slices) : m_e.shape()[index - newaxis_count_before<S...>(index)];
         }
     }
     //@}
@@ -341,10 +342,28 @@ namespace xt
     template <class E>
     inline auto xview<CT, S...>::operator=(const xexpression<E>& e) -> self_type&
     {
-        return semantic_base::operator=(e);
+        bool cond = (e.derived_cast().shape().size() == dimension())
+                    && std::equal(shape().begin(), shape().end(), e.derived_cast().shape().begin());
+        if(!cond)
+        {
+            semantic_base::operator=(broadcast(e.derived_cast(), shape()));
+        }
+        else
+        {
+            semantic_base::operator=(e);
+        }
+        return *this;
     }
     //@}
 
+    template <class CT, class... S>
+    template <class E>
+    inline auto xview<CT, S...>::operator=(const E& e) -> disable_xexpression<E, self_type>&
+    {
+        std::fill(begin(), end(), e);
+        return *this;
+    }
+
     /**
      * @name Size and shape
      */
@@ -391,9 +410,9 @@ namespace xt
     template <class... Args>
     inline auto xview<CT, S...>::operator()(Args... args) -> reference
     {
-        return access_impl(std::make_index_sequence<sizeof...(Args)
-                                                    + integral_count<S...>()
-                                                    - newaxis_count<S...>()>(),
+        return access_impl(std::make_index_sequence<(sizeof...(Args) + integral_count<S...>() > newaxis_count<S...>() ?
+                                                        sizeof...(Args) + integral_count<S...>() - newaxis_count<S...>() :
+                                                        0)>(),
                            args...);
     }
 
@@ -421,12 +440,12 @@ namespace xt
     template <class... Args>
     inline auto xview<CT, S...>::operator()(Args... args) const -> const_reference
     {
-        return access_impl(std::make_index_sequence<sizeof...(Args)
-                                                    + integral_count<S...>()
-                                                    - newaxis_count<S...>()>(),
+        return access_impl(std::make_index_sequence<(sizeof...(Args) + integral_count<S...>() > newaxis_count<S...>() ?
+                                                        sizeof...(Args) + integral_count<S...>() - newaxis_count<S...>() :
+                                                        0)>(),
                            args...);
     }
-    
+
     template <class CT, class... S>
     inline auto xview<CT, S...>::operator[](const xindex& index) const -> const_reference
     {
@@ -555,7 +574,7 @@ namespace xt
     template <class CT, class... S>
     inline void xview<CT, S...>::assign_temporary_impl(temporary_type& tmp)
     {
-        std::copy(tmp.storage_cbegin(), tmp.storage_cend(), begin());
+        std::copy(tmp.cbegin(), tmp.cend(), xbegin());
     }
 
     namespace detail
@@ -590,21 +609,90 @@ namespace xt
         return detail::make_view_impl(std::forward<E>(e), std::make_index_sequence<sizeof...(S)>(), std::forward<S>(slices)...);
     }
 
-    /****************
-     * iterator api *
-     ****************/
+    /***************
+    * iterator api *
+    ****************/
 
     /**
-     * @name Iterators
+    * @name Iterators
+    */
+    //@{
+    /**
+     * Returns an iterator to the first element of the buffer containing
+     * the elements of the view.
+     */
+    template <class CT, class... S>
+    inline auto xview<CT, S...>::begin() -> iterator
+    {
+        return xbegin();
+    }
+
+    /**
+     * Returns an iterator to the element following the last element of
+     * the buffer containing the elements of the view.
+     */
+    template <class CT, class... S>
+    inline auto xview<CT, S...>::end() -> iterator
+    {
+        return xend();
+    }
+
+    /**
+     * Returns a constant iterator to the first element of the buffer
+     * containing the elements of the view.
+     */
+    template <class CT, class... S>
+    inline auto xview<CT, S...>::begin() const -> const_iterator
+    {
+        return xbegin();
+    }
+
+    /**
+     * Returns a constant iterator to the element following the last
+     * element of the buffer containing the elements of the view.
+     */
+    template <class CT, class... S>
+    inline auto xview<CT, S...>::end() const -> const_iterator
+    {
+        return xend();
+    }
+
+    /**
+     * Returns a constant iterator to the first element of the buffer
+     * containing the elements of the view.
+     */
+    template <class CT, class... S>
+    inline auto xview<CT, S...>::cbegin() const -> const_iterator
+    {
+        return cxbegin();
+    }
+
+    /**
+     * Returns a constant iterator to the element following the last
+     * element of the buffer containing the elements of the view.
+     */
+    template <class CT, class... S>
+    inline auto xview<CT, S...>::cend() const -> const_iterator
+    {
+        return cxend();
+    }
+    //@}
+
+    /**************************
+     * braodcast iterator api *
+     **************************/
+
+    /**
+     * @name Broadcast iterators
      */
     //@{
     /**
      * Returns an iterator to the first element of the view.
      */
     template <class CT, class... S>
-    inline auto xview<CT, S...>::begin() -> iterator
+    inline auto xview<CT, S...>::xbegin() -> broadcast_iterator
     {
-        return xbegin(shape());
+        return broadcast_iterator(stepper_begin(m_shape), &m_shape);
     }
 
     /**
@@ -612,18 +700,18 @@ namespace xt
      * of the view.
      */
     template <class CT, class... S>
-    inline auto xview<CT, S...>::end() -> iterator
+    inline auto xview<CT, S...>::xend() -> broadcast_iterator
     {
-        return xend(shape());
+        return broadcast_iterator(stepper_end(m_shape), &m_shape);
     }
 
     /**
      * Returns a constant iterator to the first element of the view.
      */
     template <class CT, class... S>
-    inline auto xview<CT, S...>::begin() const -> const_iterator
+    inline auto xview<CT, S...>::xbegin() const -> const_broadcast_iterator
     {
-        return xbegin(shape());
+        return const_broadcast_iterator(stepper_begin(m_shape), &m_shape);
     }
 
     /**
@@ -631,18 +719,18 @@ namespace xt
      * of the view.
      */
     template <class CT, class... S>
-    inline auto xview<CT, S...>::end() const -> const_iterator
+    inline auto xview<CT, S...>::xend() const -> const_broadcast_iterator
     {
-        return xend(shape());
+        return const_broadcast_iterator(stepper_end(m_shape), &m_shape);
     }
 
     /**
      * Returns a constant iterator to the first element of the view.
      */
     template <class CT, class... S>
-    inline auto xview<CT, S...>::cbegin() const -> const_iterator
+    inline auto xview<CT, S...>::cxbegin() const -> const_broadcast_iterator
     {
-        return begin();
+        return xbegin();
     }
 
     /**
@@ -650,9 +738,9 @@ namespace xt
      * of the view.
      */
     template <class CT, class... S>
-    inline auto xview<CT, S...>::cend() const -> const_iterator
+    inline auto xview<CT, S...>::cxend() const -> const_broadcast_iterator
     {
-        return end();
+        return xend();
     }
 
     /**
@@ -766,75 +854,6 @@ namespace xt
         return const_stepper(this, e.stepper_end(m_e.shape()), offset, true);
     }
 
-    /************************
-     * storage_iterator api *
-     ************************/
-
-    /**
-     * @name Storage iterators
-     */
-    //@{
-    /**
-     * Returns an iterator to the first element of the buffer containing
-     * the elements of the view.
-     */
-    template <class CT, class... S>
-    inline auto xview<CT, S...>::storage_begin() -> storage_iterator
-    {
-        return begin();
-    }
-
-    /**
-     * Returns an iterator to the element following the last element of
-     * the buffer containing the elements of the view.
-     */
-    template <class CT, class... S>
-    inline auto xview<CT, S...>::storage_end() -> storage_iterator
-    {
-        return end();
-    }
-
-    /**
-     * Returns a constant iterator to the first element of the buffer
-     * containing the elements of the view.
-     */
-    template <class CT, class... S>
-    inline auto xview<CT, S...>::storage_begin() const -> const_storage_iterator
-    {
-        return cbegin();
-    }
-
-    /**
-     * Returns a constant iterator to the element following the last
-     * element of the buffer containing the elements of the view.
-     */
-    template <class CT, class... S>
-    inline auto xview<CT, S...>::storage_end() const -> const_storage_iterator
-    {
-        return cend();
-    }
-
-    /**
-     * Returns a constant iterator to the first element of the buffer
-     * containing the elements of the view.
-     */
-    template <class CT, class... S>
-    inline auto xview<CT, S...>::storage_cbegin() const -> const_storage_iterator
-    {
-        return cbegin();
-    }
-
-    /**
-     * Returns a constant iterator to the element following the last
-     * element of the buffer containing the elements of the view.
-     */
-    template <class CT, class... S>
-    inline auto xview<CT, S...>::storage_cend() const -> const_storage_iterator
-    {
-        return cend();
-    }
-    //@}
-
     /********************************
      * xview_stepper implementation *
      ********************************/
